         program gid1di
         implicit real*8(a-h,o-z)
c        Main program for 1D implicit schemes
c        for Lagrangian gas dynamics.

c        Number of nodes, time and time step
         common/nm/ n
         common/tdt/ t, dt

         n=101
c       
c        Maximum time
         tmax=0.501

         dt=0.001


c        Set t and tout,dtout

         t=0.
         dtout=0.5
         tout=dtout
 

   
c         Computation of the initial values
c         for parameters and some useful
c         arrays as the mass of cells and nodes.

          call ini
          
c         Computation of parameters on the next step

1         continue

          t=t+dt

          call stepi

          if(t.ge.tmax) stop
        
c         Output of the results  

          if(t.gt.tout) then

          call output
          tout=tout+dtout
          end if

          go to 1

          end

c================================================
        subroutine ini
        implicit real*8(a-h,o-z)
c       Computation of the initial data
c       and useful arrays
c        x - coordinate
c        w - velocity
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cells
c        zmn - mass of nodes
c        zmc - mass of cells

        common/nm/ n

        common/x/ x(0:201)
        common/rho/ rho(0:201)
        common/vareps/ e(0:201)
        common/press/ p(0:201)
        common/wx/ w(0:201)
        common/mass/ zmc(0:201), zmn(0:201)
        common/vol/ vc(0:201), vn(0:201)

        
c       Grid 
        h=1./(n-1.)
        do i=1,n
        x(i)=(i-1.)*h
        end do
c       
c       Fictitious nodes 0 and n+1
        x(0)=x(1)
        x(n+1)=x(n)

        
c       Initial profile for density and internal energy
c       and pressure
        do i=1,n-1
        xm=0.5*(x(i+1)+x(i))
        rho(i)=frho(xm)
        e(i)=fe(xm)
c       equation of state
        p(i)=fp(rho(i),e(i))
        end do

c       Initial profile for velocity
        do i=2,n-1
        w(i)=fw(x(i))
        end do
c       Boundary conditions for velocity
c       Piston problem 
c       left boundary is the surface of the piston
c       and moving with constant velocity
c       w(1)=1.                     
c       right boundary is a stationary wall
c       w(n)=0.

        w(1)=1.
        w(n)=0.

c       Volume of the cells
        do i=0,n
        vc(i)=x(i+1)-x(i)
        end do

c       Mass of the cells
        do i=0,n
        zmc(i)=rho(i)*vc(i)
        end do

c       Mass of the nodes
        do i=1,n
        zmn(i)=0.5*(zmc(i-1)+zmc(i))
        end do

        return
        end 

c==================================================

         real function fp*8(rho,e)
        implicit real*8(a-h,o-z)
c       Equation of state
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        fp = (gamma-1.)*rho*e

        return
        end    

c ==================================================
         real function ff*8(p,rho,e)
          implicit real*8(a-h,o-z)
c       Equation of state in form f(p,rho,e)=0
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        ff = p-(gamma-1.)*rho*e

        return
        end    
c ==================================================
         real function fdfdrho*8(p,rho,e)
          implicit real*8(a-h,o-z)
c       derivative df/drho
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        fdfdrho = -(gamma-1.)*e

        return
        end    
c ==================================================
         real function fdfdp*8(p,rho,e)
          implicit real*8(a-h,o-z)
c       derivative df/dp
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        fdfdp = 1.             

        return
        end    
c ==================================================
         real function fdfde*8(p,rho,e)
          implicit real*8(a-h,o-z)
c       derivative df/de
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        fdfde = -(gamma-1.)*rho

        return
        end    
c ==================================================
         real function fc*8(rho,e)
          implicit real*8(a-h,o-z)
c        Adiabatic speed of sound

         gamma=5./3.
         fc=sqrt((gamma-1.)*gamma*e)

        return
         end
c ==================================================

         real function frho*8(x)
          implicit real*8(a-h,o-z)
c       Initial profile for density
        frho = 1.
        return
        end

c ==================================================

         real function fe*8(x)
          implicit real*8(a-h,o-z)
c       Initial profile for internal energy
        fe = 0.0001     
        return
         end

c ==================================================

         real function fw*8(x)
          implicit real*8(a-h,o-z)

c        Initial profile for velocity
        fw = 0.0      
        return
         end
c ==================================================

         subroutine stepi
c        implicit 1D hydrodynamics
c        artificial viscosity is explicit

         implicit real*8(a-h,o-z)
       
c        x - coordinate
c        w - velocity
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cells
c        zmn - mass of nodes
c        zmc - mass of cells

        common/nm/ n
        common/tdt/ t, dt

        common/x/ x(0:201)
        common/rho/ rho(0:201)
        common/vareps/ e(0:201)
        common/press/ p(0:201)
        common/wx/ w(0:201)
        common/mass/ zmc(0:201), zmn(0:201)
        common/vol/ vc(0:201), vn(0:201)

c       Velocity on n+1 - np1 time level
        dimension wnp1(0:201)
c       Coordinates on n+1 - np1 time level
        dimension xnp1(0:201)
c       Energy on n+1 - np1 time level
        dimension enp1(0:201)

c         (s+1)
c       dp     - pressure correction
        dimension dp(0:201)
        
c        Denominator in the formula
c        for the central coefficient
c        of the system of linear equations
         dimension zna(0:201)

c       Coefficients of the system of linear equations
c       AM1(i)*dp(i-1) + AC(i)*dp(i) + AP1(i)*dp(i+1) =  f(i)
        dimension AM1(0:201),AC(0:201),AP1(0:201),f(0:201)
       
c       Artificial viscosity
        dimension q(0:201)
   
c       Tolerance for linear and non-linear iterations
        epsl=0.0001
        epsn=0.0001     
     
c       Time weights

        sigma1=0.5     
        sigma2=0.5     
        sigma3=0.5     
  

c       Artificial viscosity
c       Linear artificial viscosity
c       Coefficient of artificial viscosity
        znu=1.5  
        gamma=5./3.
        do i=1,n-1
        if(w(i+1)-w(i).ge.0) then
        q(i)=0.
        else
        zfc=fc(rho(i), e(i))
        q(i)=-znu*rho(i)* zfc*(w(i+1)-w(i))
c       Quadratic viscosity
c       q(i)= znu*rho(i)*(w(i+1)-w(i))**2
        end if
        end do

        
c       Boundary conditions for 
c       artificial viscosity

        q(0)=q(1)
        q(n)=q(n-1)

c
c       EXPLICIT artificial viscosity
              
c       New velocities

c       boundary conditions
        wnp1(1) = 1.0
        wnp1(n) = 0.
c
c       internal nodes from the equation of motion
c       
        do i=2,n-1
        wnp1(i) = w(i)-dt*(q(i)-q(i-1))/zmn(i)
        end do

c       New internal energy
        do i=1,n-1
        e(i)=e(i)-dt*q(i)*
     *  (0.5*(wnp1(i+1)+w(i+1))-0.5*(wnp1(i)+w(i)))/zmc(i)
        end do

        do i=2,n-1
        w(i) = wnp1(i)
        end do
 
        
c        Coefficients of the system of linear equations
c        are not changing during iterations  
c        and are computed using the parameters from the previous time step

         do i=1,n-1
         if(i.eq.1) then
         AM1(i)=0.
         else
         AM1(i)=-1./zmn(i)
         end if
         if(i.eq.n-1)then
         AP1(i)=0.
         else
         AP1(i)=-1./zmn(i+1)
         end if
         dfdp=fdfdp(p(i),rho(i),e(i))
         dfdrho=fdfdrho(p(i),rho(i),e(i))
         dfde=fdfde(p(i),rho(i),e(i))
         zna(i)=
     *   (zmc(i)*dfdrho*sigma*sigma2*dt**2/(vc(i)**2)+
     *    dfde*p(i)*sigma1*sigma3*dt**2/zmc(i))
         AC(i)=(1./zmn(i)+1./zmn(i+1))+
     *   (-dfdp+dfde*sigma1*dt*(w(i+1)-w(i))/zmc(i))/zna(i)
         end do
   
c        Beginning of non-linear iterations.
c        Method of parallel chord.
c        Initial guess for pressure
c        is the values from the previous time step

1       continue

c       boundary conditions
        wnp1(1) = 1.0
        wnp1(n) = 0.
c
c       internal nodes from the equation of motion
c       
        do i=2,n-1
        wnp1(i) = w(i)-dt*(p(i)-p(i-1))/zmn(i)
        end do

c       New coordinates
        do i=1,n
        xnp1(i)=x(i)+0.5*dt*(wnp1(i)+w(i))
        end do

c       New internal energy
        do i=1,n-1
        enp1(i)=e(i)-dt*p(i)*
     *  (0.5*(wnp1(i+1)+w(i+1))-0.5*(wnp1(i)+w(i)))/zmc(i)
        end do

 
c       New volume
        do i=1,n-1
        vc(i)=xnp1(i+1)-xnp1(i)
        end do


c       Computation of the residual
c       and its maximum norm
        zmaxn=0.
        do i=1,n-1
        f(i)=ff(p(i),rho(i),e(i))/zna(i)
        res=ff(p(i),rho(i),e(i))
        zmaxn=amax1(zmaxn,abs(res))
        end do

c       New density
        do i=1,n-1
        rho(i)=zmc(i)/vc(i)
        end do

c       Checking convergence of
c       non-linear iterations
        if(zmaxn.ge.epsn) then

c       Solution for the system of linear equations
c       to find the correction to pressure - dp(i)

      
c        Start of the Siedel iteration to
c        solve the system of linear equations

c        Initial guess for dp
         do i=1,n-1
         dp(i)=0.
         end do

2        continue

         zmaxl=0.
         do i=1,n-1
         dpi=(f(i)-AM1(I)*dp(i-1)-AP1(i)*dp(i+1))/AC(i)
         zmaxl= max1(zmax1,abs(dpi-dp(i)))
         dp(i)=dpi
         end do

         if(zmaxl.ge.epsl) go to 2

c        Pressure on the new iteration
         do i=1,n-1
         p(i)=p(I)+dp(i)
         end do
         go to 1
         end if


c        Values of parameters on the new time level

         do i=1,n
         w(i)=wnp1(i)
         x(i)=xnp1(i)
         end do

         do i=1,n-1
         e(i)=enp1(i)
         end do
         
         return 
         end
c =====================================================
      
         subroutine output
         implicit real*8(a-h,o-z)
c        Output of flow parameters
 
c        x - coordinate
c        w - velocity
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cells
c        zmn - mass of nodes
c        zmc - mass of cells
 
        common/nm/ n
        common/tdt/ t, dt
 
        common/x/ x(0:201)
        common/rho/ rho(0:201)
        common/vareps/ e(0:201)
        common/press/ p(0:201)
        common/wx/ w(0:201)
        common/mass/ zmc(0:201), zmn(0:201)
 
        common/vol/ vc(0:201), vn(0:201)
 
        open(unit=1,file='g1di.output')
 
        write(1,*) '               This is test output for program g1di'
        write(1,*)
        write(1,*) '        n=',n,'    t=',t,'     dt=',dt
        write(1,*)
 
        write(1,88)
88      format(2X,'i',10X,'x',13X,'w',12X,'rho',14X,'e',13X,'p')

        do i=1,n
        write(1,77) i,x(i),w(i),rho(i),e(i),p(i)
        end do
 
77      format(1X,i3,5(2X,e13.6))
 
        return
        end
 
 



