         program gid2di
         implicit real*8(a-h,o-z)
c        Main program for 2D implicit schemes
c        for Lagrangian gas dynamics.

c        Number of nodes, time and time step

         common/mn/ m,n
         common/tdt/ t, dt

         m=101
         n=11

c        maximum time

         tmax=0.501

         dt=0.001


c        Set t and tout,dtout

         t=0.
         dtout=0.5
         tout=dtout
 

   
c         Computation of the initial values
c         for the parameters and some useful
c         arrays as a mass of cells and nodes.

          call ini
          
c         Computation of parameters on the next step

1         continue
          t=t+dt

          call stepi

          if(t.ge.tmax) stop
        
c         Output of results  

          if(t.ge.tout) then
          call output
          tout=tout+dtout
          end if

          go to 1

          end

c================================================

        subroutine ini
        implicit real*8(a-h,o-z)
c       Computation of the initial data
c       and useful arrays
c        x,y - coordinate
c        wx,wy - velocity
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cells
c        zmn - mass of nodes
c        zmc - mass of cells

        common/mn/ m,n

        common/xy/ x(0:201,0:201), y(0:201,0:201)
        common/rho/ rho(0:201,0:201)
        common/vareps/ e(0:201,0:201)
        common/press/ p(0:201,0:201)
        common/wxwy/ wx(0:201,0:201), wy(0:201,0:201)
        common/mass/ zmc(0:201,0:201), zmn(0:201,0:201)
        common/vol/ vc(0:201,0:201), vn(0:201,0:201)

 
c       Grid
        hx=1./(m-1.)
        hy=0.1/(n-1.)
        pi=acos(-1.)
 
        do i=1,m
        do j=1,n
        x(i,j)=hx*(i-1.)+(n-j)*hx*sin(pi*(i-1.)/(m-1.))
        y(i,j)=hy*(j-1.)
        end do
        end do
c
c       Fictitious nodes
        do i=1,m
        x(i,0)=x(i,1)
        x(i,n+1)=x(i,n)
        y(i,0)=y(i,1)
        y(i,n+1)=y(i,n)
        end do
        do j=1,n
        x(0,j)=x(1,j)
        x(m+1,j)=x(m,j)
        y(0,j)=y(1,j)
        y(m+1,j)=y(m,j)
        end do
 
 
c       Initial profile for density and internal energy
c       and pressure
        do i=1,m-1
        do j=1,n-1
        xm=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
        rho(i,j)=frho(xm)
        e(i,j)=fe(xm)
c       equation of state
        p(i,j)=fp(rho(i,j),e(i,j))
        end do
        end do
 
c       Initial profile for velocity
        do i=2,m-1
        do j=1,n
        wx(i,j)=fw(x(i,j))
        wy(i,j)=0.
        end do
        end do
 
c       Boundary conditions for velocity
c       Piston problem
c       left boundary is the surface of the piston
c       and moving with constant velocity
c       w(1,j)=1.                    
c       right boundary is a stationary wall
c       w(m,j)=0.
        do j=1,n
        wx(1,j)=1.
        wx(m,j)=0.
 
        wy(1,j)=0.
        wy(m,j)=0.
        end do
 
        do i=1,m
        wy(i,1)=0.
        wy(i,n)=0.
        end do
 
 
 
c        Boundary conditions for pressure
 
c        Top and bottom boundary
c        stationary walls
         do i=1,m-1
         p(i,0)=p(i,1)
         p(i,n)=p(i,n-1)
         end do
 
c        Right boundary
c        stationary wall
         do j=1,n-1
         p(m,j)=p(m-1,j)
         end do
c        Left boundary
c        Piston which is moving with CONSTANT velocity
         do j=1,n-1
         p(0,j)=p(1,j)
         end do
c
 
c       Volume of the cells
        do i=0,m
        do j=0,n
        vc(i,j)=0.5*((x(i+1,j)-x(i,j+1))*(y(i+1,j+1)-y(i,j))-
     *               (y(i+1,j)-y(i,j+1))*(x(i+1,j+1)-x(i,j)))
        end do
        end do
 
 
c       Mass of the cells
        do i=0,m
        do j=0,n
        zmc(i,j)=rho(i,j)*vc(i,j)
        end do
        end do
 
c       Mass of the nodes
        do i=1,m
        do j=1,n
        zmn(i,j)=0.25*(zmc(i-1,j-1)+zmc(i,j-1)+zmc(i,j)+zmc(i-1,j))
        end do
        end do

        return
        end 

c==================================================

         real function fp*8(rho,e)
          implicit real*8(a-h,o-z)
c       Equation of state
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        fp = (gamma-1.)*rho*e

        return
        end    

c ==================================================
         real function ff*8(p,rho,e)
          implicit real*8(a-h,o-z)
c       Equation of state in form f(p,rho,e)=0
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        ff = p-(gamma-1.)*rho*e

        return
        end    
c ==================================================
         real function fdfdrho*8(p,rho,e)
          implicit real*8(a-h,o-z)
c       derivative df/drho
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        fdfdrho = -(gamma-1.)*e

        return
        end    
c ==================================================
         real function fdfdp*8(p,rho,e)
          implicit real*8(a-h,o-z)
c       derivative df/dp
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        fdfdp = 1.             

        return
        end    
c ==================================================
         real function fdfde*8(p,rho,e)
          implicit real*8(a-h,o-z)
c       derivative df/de
c       (Ideal gas gamma=5/3)

        gamma=5./3.

        fdfde = -(gamma-1.)*rho

        return
        end    
c ==================================================
         real function fc*8(rho,e)
          implicit real*8(a-h,o-z)
c        Adiabatic speed of sound

         gamma=5./3.
         fc=sqrt((gamma-1.)*gamma*e)

        return
         end
c ==================================================

         real function frho*8(x)
          implicit real*8(a-h,o-z)
c       Initial profile for density
        frho = 1.
        return
        end

c ==================================================

         real function fe*8(x)
          implicit real*8(a-h,o-z)
c       Initial profile for internal energy
        fe = 0.0001      
        return
         end

c ==================================================

         real function fw*8(x)
          implicit real*8(a-h,o-z)

c        Initial profile for velocity
        fw = 0.0      
        return
         end
c ==================================================

         real function fdvdx*8(i,j,k,l)
c        derivative of volume
c        dV(i,j)/dx(k,l)
 
c        for (k,l) = (i,j)
c            (k,l) = (i+1,j)
c            (k,l) = (i+1,j+1)
c            (k,l) = (i,j+1)
 
         implicit real*8(a-h,o-z)
         common/xy/ x(0:201,0:201), y(0:201,0:201)
 
         if(k.eq.i.and.l.eq.j) then
         fdvdx=0.5*(y(i+1,j)-y(i,j+1))
         end if
         if(k.eq.i+1.and.l.eq.j) then
         fdvdx=-0.5*(y(i,j)-y(i+1,j+1))
         end if
         if(k.eq.i+1.and.l.eq.j+1) then
         fdvdx=-0.5*(y(i+1,j)-y(i,j+1))
         end if
         if(k.eq.i.and.l.eq.j+1) then
         fdvdx=0.5*(y(i,j)-y(i+1,j+1))
 
         end if
 
        return
         end
c ==================================================
         real function fdvdy*8(i,j,k,l)
c        derivative of volume
c        dV(i,j)/dy(k,l)
 
c        for (k,l) = (i,j)
c            (k,l) = (i+1,j)
c            (k,l) = (i+1,j+1)
c            (k,l) = (i,j+1)
 
         implicit real*8(a-h,o-z)
         common/xy/ x(0:201,0:201), y(0:201,0:201)
 
         if(k.eq.i.and.l.eq.j) then
         fdvdy=-0.5*(x(i+1,j)-x(i,j+1))
         end if
         if(k.eq.i+1.and.l.eq.j) then
         fdvdy=0.5*(x(i,j)-x(i+1,j+1))
         end if
 
         if(k.eq.i+1.and.l.eq.j+1) then
         fdvdy=0.5*(x(i+1,j)-x(i,j+1))
         end if
         if(k.eq.i.and.l.eq.j+1) then
         fdvdy=-0.5*(x(i,j)-x(i+1,j+1))
         end if
 
        return
         end
c ==========================================================

         subroutine stepi
c        implicit 2D hydrodynamics

         implicit real*8(a-h,o-z)
       
c        x,y - coordinate
c        wx,wy - velocity
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cells
c        zmn - mass of nodes
c        zmc - mass of cells

        common/mn/ m,n
        common/tdt/ t, dt

        common/xy/ x(0:201,0:201), y(0:201,0:201)
        common/rho/ rho(0:201,0:201)
        common/vareps/ e(0:201,0:201)
        common/press/ p(0:201,0:201)
        common/wxwy/ wx(0:201,0:201),wy(0:201,0:201)
        common/mass/ zmc(0:201,0:201), zmn(0:201,0:201)
        common/vol/ vc(0:201,0:201), vn(0:201,0:201)

c       Velocity on n+1 - np1 time level
        dimension wxnp1(0:201,0:201),wynp1(0:201,0:201)
c       Coordinates on n+1 - np1 time level
        dimension xnp1(0:201,0:201), ynp1(0:201,0:201)
c       Energy on n+1 - np1 time level
        dimension enp1(0:201,0:201)
c         (s+1)
c       dp     - pressure correction
        dimension dp(0:201,0:201)
        
c        Denominator
         dimension zna(0:201,0:201)

c       Coefficients of the system of linear equations
c        AIM1JM1(i,j)*dp(i-1,j-1)+ AIJM1(i,j)*dp(i,j-1)+
c        AIP1JM1(i,j)*dp(i+1,j-1)+AIP1J(i,j)*dp(i+1,j)+
c        AIP1JP1(i,j)*dp(i+1,j+1)+AIJP1(i,j)*dp(i,j+1)+
c        AIM1JP1(i,j)*dp(i-1,j+1)+AIM1J(i,j)*dp(i-1,j)
c        +AC(i,j)*dp(i,j)=f(i,j)
        dimension AIM1JM1(0:201,0:201), AIJM1(0:201,0:201),
     *            AIP1JM1(0:201,0:201), AIP1J(0:201,0:201),
     *            AIP1JP1(0:201,0:201), AIJP1(0:201,0:201),
     *            AIM1JP1(0:201,0:201), AIM1J(0:201,0:201),
     *            AC(0:201,0:201), f(0:201,0:201)
       
c       Artificial viscosity
        dimension q(0:201,0:201)
   
c       Tolerance for linear and non-linear iterations
        epsl=0.0001
        epsn=0.0001     
     
c       Weights

        sigma1=0.5     
        sigma2=0.5     
        sigma3=0.5     
  
     
 
c       Artificial viscosity
c       Linear artificial viscosity
c       Coefficient of artificial viscosity
        znu=1. 
        gamma=5./3.
        do i=1,m-1
        do j=1,n-1
c       Divergence of the velocity
 
c       from n-time level
        DIVij=(1./vc(i,j))*
     *  (fdvdx(i,j,i,j)*wx(i,j)+fdvdx(i,j,i+1,j)*wx(i+1,j)+
     *   fdvdx(i,j,i+1,j+1)*wx(i+1,j+1)+fdvdx(i,j,i,j+1)*wx(i,j+1)+
     *   fdvdy(i,j,i,j)*wy(i,j)+fdvdy(i,j,i+1,j)*wy(i+1,j)+
     *   fdvdy(i,j,i+1,j+1)*wy(i+1,j+1)+fdvdy(i,j,i,j+1)*wy(i,j+1))
        if(DIVij.ge.0) then
        q(i,j)=0.
        else
        zfc=fc(rho(i,j), e(i,j))
        zl=sqrt(vc(i,j))
        q(i,j)=-znu*rho(i,j)* zfc*zl*DIVij
        end if
        end do
        end do
c       Boundary conditions for artificial viscosity
c       the same as for pressure
 
c        Top and bottom boundary
c        stationary walls
         do i=1,m-1
 
         q(i,0)=q(i,1)
         q(i,n)=q(i,n-1)
         end do     
 
c        Right boundary
c        stationary wall
         do j=1,n-1
         q(m,j)=q(m-1,j)
         end do
c        Left boundary
c        Piston which is moving with CONSTANT velocity
         do j=1,m-1
         q(0,j)=q(1,j)
         end do
 
c      New velocities
 
c       Boundary conditions
 
        do j=1,n
        wxnp1(1,j)=1.
        wxnp1(m,j)=0.
 
        end do
        do i=1,m
        wynp1(i,1)=0.
        wynp1(i,n)=0.
        end do
c       internal nodes from the equation of motion
c       explicit viscosity
        do i=2,m-1
        do j=1,n
        zpij=q(i,j)
        zpim1j=q(i-1,j)
        zpim1jm1=q(i-1,j-1)
        zpijm1=q(i,j-1)
        wxnp1(i,j) = wx(i,j)+
     *  dt*(fdvdx(i,j,i,j)*zpij+fdvdx(i-1,j,i,j)*zpim1j+
     *      fdvdx(i-1,j-1,i,j)*zpim1jm1+fdvdx(i,j-1,i,j)*zpijm1)
     *   /zmn(i,j)
        end do
 
        end do
 
        do i=1,m
        do j=2,n-1
        zpij=q(i,j)
        zpim1j=q(i-1,j)
        zpim1jm1=q(i-1,j-1)
        zpijm1=q(i,j-1)
        wynp1(i,j) = wy(i,j)+
     *  dt*(fdvdy(i,j,i,j)*zpij+fdvdy(i-1,j,i,j)*zpim1j+
     *      fdvdy(i-1,j-1,i,j)*zpim1jm1+fdvdy(i,j-1,i,j)*zpijm1)
     *   /zmn(i,j)
        end do
        end do

c

c       New internal energy
        do i=1,m-1
        do j=1,n-1
        wxij=0.5*(wxnp1(i,j)+wx(i,j))
        wyij=0.5*(wynp1(i,j)+wy(i,j))
        wxip1j=0.5*(wxnp1(i+1,j)+wx(i+1,j))
        wyip1j=0.5*(wynp1(i+1,j)+wy(i+1,j))
        wxip1jp1=0.5*(wxnp1(i+1,j+1)+wx(i+1,j+1))
        wyip1jp1=0.5*(wynp1(i+1,j+1)+wy(i+1,j+1))
 
       wxijp1=0.5*(wxnp1(i,j+1)+wx(i,j+1))
        wyijp1=0.5*(wynp1(i,j+1)+wy(i,j+1))
        e(i,j)=e(i,j)-
     *  dt*q(i,j)*
     *  (fdvdx(i,j,i,j)*wxij+fdvdx(i,j,i+1,j)*wxip1j+
     *   fdvdx(i,j,i+1,j+1)*wxip1jp1+fdvdx(i,j,i,j+1)*wxijp1+
     *   fdvdy(i,j,i,j)*wyij+fdvdy(i,j,i+1,j)*wyip1j+
     *   fdvdy(i,j,i+1,j+1)*wyip1jp1+fdvdy(i,j,i,j+1)*wyijp1)
     *  /zmc(i,j)
        end do
        end do

         do i=1,m
         do j=1,n
         wx(i,j)=wxnp1(i,j)
         wy(i,j)=wynp1(i,j)
         end do
         end do
 
        
c       Initial guess for pressure
c       is the values from the previous time step.
c       Coefficients of the system of linear equations
c       are not changing during iterations  
c       and are computed using parameters from the previous time step

         do i=1,m-1
         do j=1,n-1
         dfdp=fdfdp(p(i,j),rho(i,j),e(i,j))
         dfdrho=fdfdrho(p(i,j),rho(i,j),e(i,j))
         dfde=fdfde(p(i,j),rho(i,j),e(i,j))
         zna(i,j)=
     *   (zmc(i,j)*dfdrho*sigma*sigma2*dt**2/(vc(i,j)**2)+
     *    dfde*p(i,j)*sigma1*sigma3*dt**2/zmc(i,j))
          sumsq=0.     
          do ial=0,1
          do jbe=0,1
          if(i+ial.eq.1.or.i+ial.eq.m) then
          sumsqx=0.
          else
          sumsqx=
     *    fdvdx(i,j,i+ial,j+jbe)**2
     *    /zmn(i+ial,j+jbe)
          end if
          if(j+jbe.eq.1.or.j+jbe.eq.n) then
          sumsqy=0.
          else
          sumsqy=
     *    fdvdy(i,j,i+ial,j+jbe)**2
     *    /zmn(i+ial,j+jbe)
          end if
          sumsq=sumsq+sumsqx+sumsqy
         end do
         end do
        Dij=
     *  (fdvdx(i,j,i,j)*wx(i,j)+fdvdx(i,j,i+1,j)*wx(i+1,j)+
     *   fdvdx(i,j,i+1,j+1)*wx(i+1,j+1)+fdvdx(i,j,i,j+1)*wx(i,j+1)+
     *   fdvdy(i,j,i,j)*wy(i,j)+fdvdy(i,j,i+1,j)*wy(i+1,j)+
     *   fdvdy(i,j,i+1,j+1)*wy(i+1,j+1)+fdvdy(i,j,i,j+1)*wy(i,j+1))
          
         AC(i,j)=sumsq+
     *   (-dfdp+dfde*sigma1*dt*Dij/zmc(i,j))/zna(i,j)

         AIP1JX=0.
         AIP1JY=0.
         do ibe=0,1
         if(i+1.eq.m) then 
         AIP1JX=AIP1JX + 0.
         else
         AIP1JX=AIP1JX +
     *   fdvdx(i,j,i+1,j+ibe)*fdvdx(i+1,j,i+1,j+ibe)/zmn(i+1,j+ibe)
         end if
         if(j+ibe.eq.1.or.j+ibe.eq.n) then
         AIP1JY=AIP1JY + 0.
         else
         AIP1JY=AIP1JY +
     *   fdvdy(i,j,i+1,j+ibe)*fdvdy(i+1,j,i+1,j+ibe)/zmn(i+1,j+ibe)
         end if
         end do
         AIP1J(i,j)=AIP1JX+AIP1JY

         AIP1JP1X=0.
         AIP1JP1Y=0.
         if(i+1.eq.m) then 
         AIP1JP1X=AIP1JP1X +0.
         else
         AIP1JP1X=AIP1JP1X +
     *   fdvdx(i,j,i+1,j+1)*fdvdx(i+1,j+1,i+1,j+1)/zmn(i+1,j+1)
         end if
         if(j+1.eq.n) then 
         AIP1JP1Y=AIP1JP1Y + 0.
         else
         AIP1JP1Y=AIP1JP1Y +
     *   fdvdy(i,j,i+1,j+1)*fdvdy(i+1,j+1,i+1,j+1)/zmn(i+1,j+1)
         end if
         AIP1JP1(i,j)=AIP1JP1X+AIP1JP1Y

         AIJP1X=0.
         AIJP1Y=0.
         do ial=0,1
         if(i+ial.eq.1.or.i+ial.eq.m) then
         AIJP1X=AIJP1X + 0.
         else
         AIJP1X=AIJP1X +
     *   fdvdx(i,j,i+ial,j+1)*fdvdx(i,j+1,i+ial,j)/zmn(i+ial,j+1)
         end if
         if(j+1.eq.n) then
         AIJP1Y=AIJP1Y + 0.
         else
         AIJP1Y=AIJP1Y +
     *   fdvdy(i,j,i+ial,j+1)*fdvdy(i,j+1,i+ial,j)/zmn(i+ial,j+1)
         end if
         end do
         AIJP1(i,j)=AIJP1X+AIJP1Y

         AIM1JP1X=0.
         AIM1JP1Y=0.
         if(i.eq.1) then
         AIM1JP1X=AIM1JP1X + 0.
         else
         AIM1JP1X=AIM1JP1X +
     *   fdvdx(i,j,i,j+1)*fdvdx(i-1,j+1,i,j+1)/zmn(i,j+1)
         end if
         if(j+1.eq.n) then
         AIM1JP1Y=AIM1JP1Y + 0.
         else
         AIM1JP1Y=AIM1JP1Y +
     *   fdvdy(i,j,i,j+1)*fdvdy(i-1,j+1,i,j+1)/zmn(i,j+1)
         end if
         AIM1JP1(i,j)=AIM1JP1X+AIM1JP1Y

         AIM1JX=0.
         AIM1JY=0.
         do ibe=0,1
         if(i.eq.1) then
         AIM1JX=AIM1JX + 0.
         else
         AIM1JX=AIM1JX +
     *   fdvdx(i,j,i,j+ibe)*fdvdx(i-1,j,i,j+ibe)/zmn(i,j+ibe)
         end if
         if(j+ibe.eq.1.or.j+ibe.eq.n) then
         AIM1JY=AIM1JY + 0.
         else
         AIM1JY=AIM1JY +
     *   fdvdy(i,j,i,j+ibe)*fdvdy(i-1,j,i,j+ibe)/zmn(i,j+ibe)
         end if
         end do
         AIM1J(i,j)=AIM1JX+AIM1JY

         AIM1JM1X=0.
         AIM1JM1Y=0.
         if(i.eq.1) then
         AIM1JM1X=AIM1JM1X + 0.
         else
         AIM1JM1X=AIM1JM1X +
     *   fdvdx(i,j,i,j)*fdvdx(i-1,j-1,i,j+1)/zmn(i,j)
         end if
         if(j.eq.1) then
         AIM1JM1Y=AIM1JM1Y + 0.
         else
         AIM1JM1Y=AIM1JM1Y +
     *   fdvdy(i,j,i,j)*fdvdy(i-1,j-1,i,j+1)/zmn(i,j)
         end if
         AIM1JM1(i,j)=AIM1JM1X+AIM1JM1Y

         AIJM1X=0.
         AIJM1Y=0.
         do ial=0,1
         if(i+ial.eq.1.or.i+ial.eq.m) then
         AIJM1X=AIJM1X + 0.
         else
         AIJM1X=AIJM1X +
     *   fdvdx(i,j,j+1,i+ial,j)*fdvdx(i,j-1,i+ial,j)/zmn(i+ial,j)
         end if
         if(j.eq.1) then
         AIJM1Y=AIJM1Y + 0.
         else
         AIJM1Y=AIJM1Y +
     *   fdvdy(i,j,j+1,i+ial,j)*fdvdy(i,j-1,i+ial,j)/zmn(i+ial,j)
         end if
         end do
         AIJM1(i,j)=AIJM1X+AIJM1Y

         AIP1JM1X=0.
         AIP1JM1Y=0.
         if(i+1.eq.m) then
         AIP1JM1X=AIP1JM1X + 0.
         else
         AIP1JM1X=AIP1JM1X +
     *   fdvdx(i,j,i+1,j)*fdvdx(i+1,j-1,i+1,j)/zmn(i+1,j)
         end if
         if(j.eq.1) then
         AIP1JM1Y=AIP1JM1Y + 0.
         else
         AIP1JM1Y=AIP1JM1Y +
     *   fdvdy(i,j,i+1,j)*fdvdy(i+1,j-1,i+1,j)/zmn(i+1,j)
         end if
         AIP1JM1(i,j)=AIP1JM1X+AIP1JM1Y

         end do
         end do
   
c        Begining of non-linear iterations
c        Method of the parallel chord

1       continue

c
c       Boundary conditions
 
        do j=1,n
        wxnp1(1,j)=1.
        wxnp1(m,j)=0.
 
        end do
        do i=1,m
        wynp1(i,1)=0.
        wynp1(i,n)=0.
        end do
     
c       internal nodes from the equation of motion
c       
        do i=2,m-1
c       do i=1,m-1
        do j=1,n
        zpij=p(i,j)
        zpim1j=p(i-1,j)
        zpim1jm1=p(i-1,j-1)
        zpijm1=p(i,j-1)
        wxnp1(i,j) = wx(i,j)+
     *  dt*(fdvdx(i,j,i,j)*zpij+fdvdx(i-1,j,i,j)*zpim1j+
     *      fdvdx(i-1,j-1,i,j)*zpim1jm1+fdvdx(i,j-1,i,j)*zpijm1)
     *   /zmn(i,j)
        end do
 
        end do
 
        do i=1,m
        do j=2,n-1
        zpij=p(i,j)
        zpim1j=p(i-1,j)
        zpim1jm1=p(i-1,j-1)
        zpijm1=p(i,j-1)
        wynp1(i,j) = wy(i,j)+
     *  dt*(fdvdy(i,j,i,j)*zpij+fdvdy(i-1,j,i,j)*zpim1j+
     *      fdvdy(i-1,j-1,i,j)*zpim1jm1+fdvdy(i,j-1,i,j)*zpijm1)
     *   /zmn(i,j)
        end do
        end do

c
c       New coordinates
 
        do i=1,m
        do j=1,n
        xnp1(i,j)=x(i,j)+0.5*dt*(wxnp1(i,j)+wx(i,j))
        ynp1(i,j)=y(i,j)+0.5*dt*(wynp1(i,j)+wy(i,j))
        end do
        end do

c       New internal energy
        do i=1,m-1
        do j=1,n-1
        wxij=0.5*(wxnp1(i,j)+wx(i,j))
        wyij=0.5*(wynp1(i,j)+wy(i,j))
        wxip1j=0.5*(wxnp1(i+1,j)+wx(i+1,j))
        wyip1j=0.5*(wynp1(i+1,j)+wy(i+1,j))
        wxip1jp1=0.5*(wxnp1(i+1,j+1)+wx(i+1,j+1))
        wyip1jp1=0.5*(wynp1(i+1,j+1)+wy(i+1,j+1))
 
       wxijp1=0.5*(wxnp1(i,j+1)+wx(i,j+1))
        wyijp1=0.5*(wynp1(i,j+1)+wy(i,j+1))
        enp1(i,j)=e(i,j)-
     *  dt*p(i,j)*
     *  (fdvdx(i,j,i,j)*wxij+fdvdx(i,j,i+1,j)*wxip1j+
     *   fdvdx(i,j,i+1,j+1)*wxip1jp1+fdvdx(i,j,i,j+1)*wxijp1+
     *   fdvdy(i,j,i,j)*wyij+fdvdy(i,j,i+1,j)*wyip1j+
     *   fdvdy(i,j,i+1,j+1)*wyip1jp1+fdvdy(i,j,i,j+1)*wyijp1)
     *  /zmc(i,j)
        end do
        end do
 
c       New volume
        do i=1,m-1
        do j=1,n-1
        vc(i,j)=0.5*((xnp1(i+1,j)-xnp1(i,j+1))*
     *               (ynp1(i+1,j+1)-ynp1(i,j))-
     *  (ynp1(i+1,j)-ynp1(i,j+1))*(xnp1(i+1,j+1)-xnp1(i,j)))
        if(vc(i,j).le.0) then
        write(*,*) 'Negative volume ', i,j
        call output
        stop
        end if
        end do
        end do


c       Computation of residual using values from the previous iteration
c       and its maximum norm
        zmaxn=0.
        do i=1,m-1
        do j=1,n-1
        f(i,j)=ff(p(i,j),rho(i,j),e(i,j))/zna(i,j)
        res=ff(p(i,j),rho(i,j),e(i,j))
        zmaxn=amax1(zmaxn,abs(res))
        end do
        end do

c        New density
         do i=1,m-1
         do j=1,n-1
         rho(i,j)=zmc(i,j)/vc(i,j)
         end do
         end do

c       Checking convergence of
c       non-linear iterations
        if(zmaxn.ge.epsn) then

c       Solution for the system of linear equations
c       to find the correction to pressure - dp(i,j)

      
c        Start of Siedel iteration to
c        solve the system of linear equations

c        Initial guess for dp
         do i=0,m
         do j=0,n
         dp(i,j)=0.
         end do
         end do

2        continue

         zmaxl=0.
         do i=1,m-1
         do j=1,n-1

         dpij = (
     *   f(i,j) 
     *  -(AIM1JM1(i,j)*dp(i-1,j-1)+ AIJM1(i,j)*dp(i,j-1)+
     *   AIP1JM1(i,j)*dp(i+1,j-1)+AIP1J(i,j)*dp(i+1,j)+
     *   AIP1JP1(i,j)*dp(i+1,j+1)+AIJP1(i,j)*dp(i,j+1)+
     *   AIM1JP1(i,j)*dp(i-1,j+1)+AIM1J(i,j)*dp(i-1,j)))
     *   /AC(i,j)
         zmaxl= max1(zmax1,abs(dpij-dp(i,j)))
         dp(i,j)=dpij
         end do
         end do
c        Boundary conditions for pressure
 
c        Top and bottom boundary
c        stationary walls
         do i=1,m-1
         dp(i,0)=dp(i,1)
         dp(i,n)=dp(i,n-1)
         end do     
 
c        Right boundary
c        stationary wall
         do j=1,n-1
         dp(m,j)=dp(m-1,j)
         end do
c        Left boundary
c        Piston which is moving with CONSTANT velocity
         do j=1,m-1
         dp(0,j)=dp(1,j)
         end do

         if(zmaxl.ge.epsl) go to 2

c           Pressure on a new iteration
         do i=1,m-1
         do j=1,n-1
         p(i,j)=p(i,j)+dp(i,j)
         end do
         end do
 
c        Boundary conditions for pressure
 
c        Top and bottom boundary
c        stationary walls
         do i=1,m-1
         p(i,0)=p(i,1)
         p(i,n)=p(i,n-1)
         end do     
 
c        Right boundary
c        stationary wall
         do j=1,n-1
         p(m,j)=p(m-1,j)
         end do
c        Left boundary
c        Piston which is moving with CONSTANT velocity
         do j=1,m-1
         p(0,j)=p(1,j)
         end do
         go to 1
         end if


c        Values of parameters on a new time level

         do i=1,m
         do j=1,n
         wx(i,j)=wxnp1(i,j)
         wy(i,j)=wynp1(i,j)
         x(i,j)=xnp1(i,j)
         y(i,j)=ynp1(i,j)
         end do
         end do

c       Fictitious nodes
        do i=1,m
        x(i,0)=x(i,1)
        x(i,n+1)=x(i,n)
        y(i,0)=y(i,1)
        y(i,n+1)=y(i,n)
        end do
        do j=1,n
        x(0,j)=x(1,j)
        x(m+1,j)=x(m,j)
        y(0,j)=y(1,j)
        y(m+1,j)=y(m,j)
        end do

         do i=1,m-1
         do j=1,n-1
         e(i,j)=enp1(i,j)
         end do
         end do
         
         return 
         end
c =====================================================
      
  
         subroutine output
         implicit real*8(a-h,o-z)
c        Output of flow parameters
c        x,y - coordinates
c        wx, wy - velocities
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cell
c        zmn - mass of nodes
c        zmc - mass of cell
 
        common/mn/ m, n
        common/tdt/ t, dt
 
        common/xy/ x(0:201,0:201), y(0:201,0:201)
        common/rho/ rho(0:201,0:201)
        common/vareps/ e(0:201,0:201)
        common/press/ p(0:201,0:201)
        common/wxwy/ wx(0:201,0:201), wy(0:201,0:201)
        common/mass/ zmc(0:201,0:201), zmn(0:201,0:201)
 
        open(unit=1,file='g2di.output')
        write(1,*) '              This is test output for program g2di'
        write(1,*)
        write(1,*) '      m=',m,' n=',n,' t=',t,' dt=',dt
        write(1,*)
 
        j=2
        write(1,*) '                                        j=',j
        write(1,*)
        write(1,*)' i        x        y          wx        wy        rho
     *           e        p'
        write(1,*)
        do i=1,m
        write(1,99) i,x(i,j),y(i,j),wx(i,j),wy(i,j),rho(i,j),
     *              e(i,j),p(i,j)
        end do
        write(1,*)
        j=6
        write(1,*) '                                        j=',j
        write(1,*)
        write(1,*)
        write(1,*)' i        x        y          wx        wy        rho
     *           e        p'
 
        do i=1,m
        write(1,99) i,x(i,j),y(i,j),wx(i,j),wy(i,j),rho(i,j),
     *              e(i,j),p(i,j)
        end do
        write(1,*)
        j=10
        write(1,*) '                                        j=',j
        write(1,*)
        write(1,*)
        write(1,*)' i        x        y          wx        wy        rho
     *           e        p'
        write(1,*)
        do i=1,m
        write(1,99) i,x(i,j),y(i,j),wx(i,j),wy(i,j),rho(i,j),
     *              e(i,j),p(i,j)
        end do
 
99      format(i3,7(1X,e10.3))
 
        return
        end
c*********************************
 
 
 



