      program h1drnc
c     This is a MAIN PROGRAM for the solution to heat equations 
c     with Robin boundary
c     value problems in the 1-D case
c
c          du/dt = d(K du/dx)/dx+f,    a < x < b
c
c      -K*(du/dx)(a,t) + alpa*u(a,t) = psia(t), at x=a,
c       K*(du/dx)(b,t) + alpb*u(b,t) = psib(t), at x=b,
c       alpa, alpb > 0.
c
c=================================================================
c    Discretization - SCALAR functions in NODES  ( HN )
c                   - VECTOR functions in CELLS ( HC )
c=================================================================
      implicit real*8 (a-h,o-z)
c================================================================

c     t - current time
      common/t/ t, dt
 
c     eps - constant in the condition of finishing an iteration in the
c     Gauss-Siedel iteration procedure.  The termination condition is
c
c              (S+1)  (S)
c     max( abs( U(i) - U(i))) < eps
c
c     where S is the iteration number.
c
      common/eps/ eps
c     IAPP - order of approximation of Robin boundary
c            conditions
      common/iapp/ IAPP

 
      common/expimp/sigma

      timemax=0.1  
 
      dtout=0.045
      tout=dtout
 
c     Set up time step and initial time
       t=0.
      dt = 0.00001    
 
c     Set type of finite-difference scheme
c     explicit  sigma=0. or implicit sigma is not equal to zero
 
c     Explicit scheme
c     sigma=0.
 
c     Implicit scheme
      sigma=0.7
c
c     ====================================================================
c     Set of coordinates of grid points
c     call subroutine mkgrid

      call mkgrid

c     Computation of geometrical quantities,
c     which do not depend on time
      call geometry
 

c     set initial conditions
      call initial
 
c     Beginning of time integration
 
1111   continue
        t = t + dt
c 
c     Set boundary conditions 
c     call subroutine bcon
      call bcon     

c     Computation of coefficient K in cells
c     call subroutine coeff
      call coeff

c     Computation of the right-hand side of the differential equation in nodes
c     call subroutine right 
      call right



c     IAPP - order of approximation of Robin boundary conditions
c     IAPP=1 - first order
c     IAPP=2 - second order
      IAPP=2
 
c     Computation of approximation for the right-hand side of the original
c     differential equation
      call sphi(IAPP)
 
c      computation of coefficients for the approximation of the elliptic 
c      operator and approximation of Robin boundary conditions
c      which is needed for explicit and implicit schemes.
      call ellcoef

c     Check Explicit or Implicit Scheme
      if(sigma.ne.0) then
 
c     Implicit scheme
c     computation of coefficients and the right-hand side of the 
c     finite-difference equations
      call hcoeff(IAPP)

c     Set of constants for the termination of the iteration process
      eps=1.d-12

c     solution for finite-difference equations by the GAUSS-SIEDEL method
      call siedel 
 
 
      else
c     Explicit scheme
 
      call step(IAPP)
 
      end if
c--------------------------------------------------------------
c      Test output
c--------------------------------------------------------------
      if(t.ge.tout) then
      write(*,*) t,tout
 
c     Set next output time
 
       tout=tout+dtout
 
c     Computing the error
      call error
 
c     Test output
      call output
 
      end if
 
      if(t.gt.timemax) stop
 
c      Go to computation of the new time level
       goto 1111
      end

c**************************************************************

      subroutine ellcoef 
c     Computation of coefficients for the discrete analog of the general
c     elliptic operator
c                    d(K du/dx)/dx
c     and discrete analog of the operator for the Robin boundary
c     conditions
c      -K*(du/dx)(a,t) + alpa*u(a,t)  at x=a,
c       K*(du/dx)(b,t) + alpb*u(b,t)  at x=b,
c       alpa, alpb > 0.
c      for 1-D case
c
c=================================================================
c     Discretization - Scalar functions in Nodes ( HN ), 
c                      Vector functions in Cells  ( HC )
c=================================================================
      implicit real*8 (a-h,o-z)
 
c     t - current time
      common/t/ t, dt
c
c     M - number of nodes in the grid
      common/M/ M
c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)
c     ZK - array (1 to M-1), value of coefficient K in cells
      common/k/ zk(200)
c     F - array (2 to M-1), values for the right-hand side of the differential
c                           equation in nodes
      common/f/ f(200)

c     alpa, alpb - constants in the left-hand side of the Robin boundary 
c     conditions in points a and b respectively.
      common/alpha/ alpa, alpb

c     psia, psib - constants in the right-hand side of the Robin boundary 
c     conditions in points a and b respectively.
      common/psi/ psia, psib

c     u -array (1 to M ), solution for the finite-difference scheme.
      common/U/ U(200)

      common/expimp/sigma
 
c     =================================================================
C     EPS - CONSTANT in condition of finishing the iteration in the 
c     Gauss-Siedel iteration procedure.  The termination condition is
c
c              (S+1)  (S)
c     max( abs( U(i) - U(i))) < eps
c
c     where S is the iteration number.
c

      common/eps/ eps

c     =================================================================
c     W, C, E, PHI - arrays of coefficients and the right-hand side
c     ( 1 to M ) in finite-difference equations
c
c                 W(i)*U(i-1) + C(i)*U(i) + E(i)*U(i+1) = PFI(i)
c
      common/coefds/ w(200),c(200),e(200),phi(200)
 
c     =================================================================
c     VC, VN - arrays for volume of cells and nodes
      common/volumes/ VC(200), VN(200)
c     =================================================================


c     computing the coefficients for the finite-difference scheme
c     and the right-hand side
c     ========================================================
      w(1) = 0.
      c(1) = zk(1)/vc(1) + alpa
      e(1) = -zk(1)/vc(1)

      do i=2,M-1
      w(i)=(-zk(i-1)/vc(i-1))/VN(i)
      c(i)=(zk(i-1)/vc(i-1)+zk(i)/vc(i))/VN(i)
      e(i)=(-zk(i)/vc(i))/VN(I)
      end do     

      w(M)=-zk(M-1)/VC(M-1)
      c(M)=zk(M-1)/VC(M-1)+alpb
      e(M) = 0.
 
       return
       end
c**************************************************************
      
       subroutine siedel
       implicit real*8 (a-h,o-z)
c=============================================================
c     M - number of nodes in the grid
      common/M/ M
c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)
c     ZK - array (1 to M-1), value of coefficient K in cells
      common/k/ zk(200)
c     F - array (2 to M-1), values of the right-hand side of the differential
c                           equation in nodes
      common/f/ f(200)

c     alpa, alpb - constants in the left-hand side of the Robin boundary 
c     conditions in points a and b respectively.
      common/alpha/ alpa, alpb

c     psia, psib - constants in the right-hand side of the Robin boundary 
c     conditions in points a and b respectively.
      common/psi/ psia, psib

c     u -array (1 to M ), solution for the finite-difference scheme.
      common/U/ U(200)
c     =================================================================
c     eps - constant in the condition of finishing the iteration in 
c     Gauss-Siedel iteration procedure.  The termination condition is
c
c              (S+1)  (S)
c     max( abs( U(i) - U(i))) < eps
c
c     where S is the iteration number.
c

      common/eps/ eps

c     =================================================================
c     W, C, E, PHI - arrays of coefficients and the right-hand side
c     ( 1 to M ) in finite-difference equations
c
c                 W(i)*U(i-1) + C(i)*U(i) + E(i)*U(i+1) = PFI(i)
c
      common/coefds/ w(200),c(200),e(200),phi(200)

c     =========================================================
c
      pi=acos(-1.)
c     SOLUTION for the system of linear algebraic equations
c     by the GAUSS-SIEDEL iteration method
c
c     Initial guess is values from the previous time step

c     beginning of the iteration process

5     continue
      EPSMAX=0.

      do i=1, M
      USP1=( phi(i)-w(i)*U(i-1)-e(i)*U(i+1))/c(i)
      EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i)))
      U(i)=USP1
      end do   

c     checking the termination condition
      if(EPSMAX.GT.EPS) goto 5
      return
      end
c*****************************************************

      subroutine mkgrid

      implicit real*8 (a-h,o-z)
c     This subroutine generates a grid in 1-D
c     That is, it generates array X (1 to M) of
c     the coordinates of the grid nodes

c     a and b coordinates of the ends of the computational domain [a,b]
      common/ab/ a,b

c     M - number of nodes in the grid
      common/M/ M

c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)

c     Set number of nodes
      M=17      

c     Set end points of the computational domain
      a=0.d0
      b=1.d0

c     Simple smooth grid as in the examples in the book

      do i=1,m
      xi=a+(b-a)*(i-1.d0)/(M-1)
      x(i)=xi**2
      end do
      return
      end
c************************************************************
      subroutine exact
      implicit real*8 (a-h,o-z)
c     Compute the exact soulution in the nodes
c     Results - array uex (1 to M)

c     M - number of nodes in the grid
      common/M/ M

c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)

c     UEX - array (1 to M) exact solution in the nodes
      common/uex/ uex(200)

c     t - current time
      common/t/ t, dt
      
       pi=acos(-1.)

      do i=1,M
       uex(i)=exp(-4.*pi**2*t)*sin(2*pi*x(i))
      end do  
      return 
      end
c************************************************************
       subroutine output
       implicit real*8 (a-h,o-z)
c      Output subroutine 

c     M - number of nodes in the grid
      common/M/ M

c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)

c     u -array (1 to M ), solution of finite-difference scheme.
      common/U/ U(200)

c     UEX - array (1 to M) exact solution in the nodes
      common/uex/ uex(200)

c     IAPP - order of approximation of Robin boundary
c            conditions
      common/iapp/ IAPP

c     ERR - array (1 to M), error between the exact
c     and approximate solutions
      common/err/ERR(200)

 
c     t - current time
      common/t/ t, dt
 
 
      common/expimp/sigma
 

c      Max-norm and L2-norm of the error between
c      the exact and approximate solutions    
       common/errors/ supn, zL2N

c       Open the file h1drnc.output for
c       test output of program H1DRNC
       open(unit=1,file='h1drnc.output')

       write(1,33)
33     format(15X, 'TEST OUTPUT FOR PROGRAM h1drnc')
c     =========================================================
      write(1,*)
      if(IAPP.eq.1) then
      write(1,44)
      else
      write(1,55)
      end if
44    format(7X, 'FIRST ORDER APPROXIMATION OF BOUNDARY CONDITIONS')
55    format(7X, 'SECOND ORDER APPROXIMATION OF BOUNDARY CONDITIONS')
      write(1,*)

      if(sigma.eq.0) then
      write(1,*)'               Explicit Scheme'
      else
      write(1,*)'               Implicit Scheme',' Sigma= ', sigma
      end if
      write(1,*)
 
 
      write(1,8) t,dt, M
      write(1,*)
      write(1,88)  supn, zL2N
8     format(2X,'Time= ',E12.5,2X,'dt = ',E12.5,2X,
     *'Number of Nodes=',i5)
88     format(2X,
     1 'MAX-NORM = ', E12.5, 2X,'  L2-NORM = ', E12.5)
       write(1,*)
 
c     Print of the approximate and exact solutions for the test problem
       write(1,9)
9      format(5X,'i',7X,' X',10X,
     1 'Appr.Sol.',4X,'Exact.Sol.',5X,'Error')
       do i=1,M
       write(1,7) i, x(i), u(i), uex(i), err(i)
       end do 
      write(1,*)
 7     format(1X, i5,2X, E12.5,2X, E12.5,2X, E12.5,2X, E12.5)
       return
       end
c**************************************************************
      subroutine error
      implicit real*8 (a-h,o-z)
c     M - number of nodes in the grid
      common/M/ M

c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)

c     VN - array ( 1 to M ) volume of nodes
c     COMMENTS - this volume is needed only for computing
c     the L2 Norm of error
      dimension VN(200)

c     u -array (1 to M ), solution for the finite-difference scheme.
      common/U/ U(200)

c     UEX - array (1 to M) exact solution in the nodes
      common/uex/ uex(200)

c      Sup-norm and L2-norm of the error between
c      the exact and approximate solutions 
       common/errors/ supn, zL2N

c     ERR - array (1 to M), error between the exact
c     and approximate solutions
      common/err/ERR(200)

 
c     VC, VN - arrays of the volume of cells and nodes
      common/volumes/ VC(200), VN(200)

c      Computing the exact solution
       call exact

c     Error
c     -----
      do i=1,M
      err(i)= uex(i)-u(i)
      end do     

c     sup - norm
c     -----------
      supn = 0.d0
      do i=1,M
      supn=amax1(supn,abs(err(i)))
      end do      

c     L  - norm
c      2
c     ----------

      zL2N = 0.d0
      do i=2, M-1
      zL2N = zL2N + (err(i))**2*VN(i)
      end do  
      zL2N=zL2N+err(1)**2+err(M)**2   
      zL2N = dsqrt(zL2N)
      return
      end
c*********************************************************************
      subroutine bcon
      implicit real*8 (a-h,o-z)

c     a and b coordinates of the ends of the computational domain [a,b]
      common/ab/ a,b

c     M - number of nodes in the grid
      common/M/ M

c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)

c     psia, psib - determine the boundary conditions in 
c     points a and b respectively at time t.
      common/psi/ psia, psib

c     alpa, alpb - coefficients near u(a,t) and u(b,t)
c     in Robin boundary conditions
      common/alpha/ alpa, alpb

c     t - current time
      common/t/ t, dt
 
      common/expimp/sigma
 
      tt=t-0.5*dt
 
      pi=acos(-1.)


      psia=fpsia(tt)
      psib=fpsib(tt)

      alpa=falpa(tt)
      alpb=falpb(tt)

      return
      end
 

c*********************************************************************

      subroutine coeff
c     In this subroutine, the coefficients of the finite-difference
c     scheme in cells must be computed
      implicit real*8 (a-h,o-z)


c     M - number of nodes in the grid
      common/M/ M

c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)

c     ZK - array (1 to M-1), value of coefficient K in cells
      common/k/ zk(200)

c     t - current time
      common/t/ t, dt
 
      common/expimp/sigma
 
      tt=t-0.5*dt

      do i=1,m-1
c     fk(x,t) - is a function which returns the values of coefficient
c     K(x,t) from the original differential equation
      zk(i)=fk(0.5*(x(i+1)+x(i)),tt)
      end do 
      return
      end
c****************************************************************

      real function fk*8(x,t)
c     fk(x,t) - is a function which returns values of coefficient
c     K(x,t) from the original differential equation
     
      implicit real*8 (a-h,o-z)
      fk=x**4 + 1.
c     fk= 1.
      return
      end
c****************************************************************

      subroutine right
c     This subroutine computes values of the right-hand side
c     of the original differential eqution in nodes

      implicit real*8 (a-h,o-z)

c     M - number of nodes in the grid
      common/M/ M

c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)


c     F - array (2 to M-1), values of the right-hand side of the differential
c                           equation in nodes
      common/f/ f(200)

c     t - current time
      common/t/ t, dt
 
      common/expimp/sigma
 
      tt=t-0.5*dt


      do i=1,m  
       f(i)=fr(x(i),tt)
c      fr(x,t) - function which returns values from the right-hand side
c                of the original differential equation in point x at
c                time moment - t.
      end do
      return 
      end
c*********************************************************************
      
      real function fr*8 (x,t)
      implicit real*8 (a-h,o-z)
c      fr(x,t) - function which returns values from the right-hand side
c                of the original differential equation in point x at
c                time moment - t.

c     Number pi=3.14...
      pi=acos(-1.)
      fr=exp(-4.*pi**2*t)*
     * (-4.*pi**2*sin(2.*pi*x)*(1.-
     * (x**4+1.))-4.*x**3*2.*pi*cos(2.*pi*x))
 
       return
       end
c**********************************************************************
 
      subroutine initial
      implicit real*8 (a-h,o-z)
c     Compute the initial values in the nodes
c     Results - array u (1 to M)
 
c     M - number of nodes in the grid
      common/M/ M
 
c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)
 
c     U -array (1 to M ), solution for the finite-difference scheme.
      common/U/ u(200)
 
c     t - current time
      common/t/ t, dt
       pi=acos(-1.)
 
c     Set up initial conditions
      t=0.
 
      do i=1,M
 
       u(i)=sin(2*pi*x(i))
      end do 
 
      return
      end
 
************************************************************
 
      subroutine step(IAPP)
      implicit real*8 (a-h,o-z)
c     This is a subroutine step for the computation of the solution to the
c     explicit finite-difference scheme for Robin    
c     boundary problems for Heat equations
c=================================================================
c     Discretization - Scalar functions in Nodes ( HN ),
c                      Vector functions in Cell  ( HC )
c=================================================================
      implicit real*8 (a-h,o-z)
 
c     t,dt- current time, and time step
 
      common/t/ t,dt
 
c     Interval of the output (in time steps)
c     and Max number of time steps
c     current time step nstep
      common/out/ nout, maxstep,nstep
 
c     M - number of nodes in the grid
      common/M/ M
 
c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)
 
c     unp1 - array (1 to M), solution on the new time level
c     U    - array (1 to M), solution on the previous time level
      common/U/  U(200)
      dimension unp1(200)
 
c     ZK - array (1 to M-1), value of coefficient K in cells
      common/k/ zk(200)
 
c     F - array (2 to M-1), values for the right-hand side of the differential
c                           equation in nodes
      common/f/ f(200)
 
c     psia, psib - determine boundary conditions in
c     points a and b respectively at time t.
      common/psi/ psia, psib
 
c     alpa, alpb - constants in the left-hand side of Robin boundary 
c     conditions in points a and b respectively.
      common/alpha/ alpa, alpb
c     =================================================================
c     W, C, E, PHI -  coefficients and the right-hand side
c     in finite-difference equations in node i
c
c                 W*U(i-1) + C*U(i) + E*U(i+1) = PFI
c
      Common/coefds/ w(200),c(200),e(200),phi(200)
c     =================================================================
 
c     VC, VN - arrays of the volume of cells and nodes
      common/volumes/ VC(200), VN(200)
c     =================================================================
 
c     computing the coefficients of the finite-difference scheme
c     and the right-hand side
 
c     ========================================================
 
c      Solution on the new time level
       do i=2,M-1
       unp1(i) = U(i)
     * + dt*(-(w(i)*U(i-1)+c(i)*U(i)+e(i)*U(i+1))
     * + phi(i))
        end do
 
c      boundary conditions
       if(IAPP.eq.1) then
c      first order approximation of boundary conditions
       unp1(1)=(psia*VC(1)+zk(1)*unp1(2))/
     *         (zk(1)+VC(1)*alpa)
       unp1(M)=(psib*VC(M-1)+zk(M-1)*unp1(M-1))/
     *         (zk(M-1)+VC(M-1)*alpb)
       else
c      second order approximation of boundary conditions
       unp1(1)=U(1)+dt*(f(1)+psia/VN(1)-
     * (-zk(1)*(U(2)-U(1))/VC(1)+alpa*U(1))/VN(1))
       unp1(M)=U(M)+dt*(f(M)+psib/VN(M)-
     * (zk(M-1)*(U(M)-U(M-1))/VC(M-1)+alpb*U(M))/VN(M))
        end if
 
c      Overdetermination of array U,
c      because we use only two arrays
c      If you need to use both arrays, do it before this 
c      overdetermination and put this loop in another subroutine
       do i=1,M
       U(i) = unp1(i)
       end do
 
      return
      end
c*************************************************************
       subroutine geometry
      implicit real*8 (a-h,o-z)
c     M - number of nodes in the grid
      common/M/ M
 
c     X - array (1 to M ), coordinates of grid nodes
      common/x/ x(200)
 
c     VC, VN - arrays of the volume of cells and nodes
      common/volumes/ VC(200), VN(200)
c     =================================================================
 
c     Computing VC and VN
 
      do 1 i=1,M-1
      VC(i) = x(i+1)-x(i)
1     continue
 
      VN(1)=0.5*(x(2)-x(1))
      do 2 i=2,M-1
      VN(i)=0.5*(x(i+1)-x(i-1))
2     continue
      VN(M)=0.5*(x(M)-x(M-1))
 
      return
      end
c*********************************************************************
      real function falpa*8(t)
      implicit real*8 (a-h,o-z)
c     falpa(t) - function which returns values of the coefficient alpha
c     in Robin boundary conditions of the original differential
c     problem, at the end point x=a, at time t.
 
       falpa= 1.
       return
       end
c*************************************************************
      real function falpb*8(t)
      implicit real*8 (a-h,o-z)
c     falpb(t) - function which returns values of the coefficient alpha
c     in Robin boundary conditions of the original differential
c     problem, at the end point x=a, at time t.
 
       falpb= 1.
       return
       end
c*************************************************************
      real function fpsia*8(t)
      implicit real*8 (a-h,o-z)
c     fpsia(t) - function which returns values of the right-hand side
c     in Robin boundary conditions of the original differential
c     problem, at the end point x=a, at time t.
c     Number pi=3.14...
      pi=acos(-1.)

       fpsia= -2.*pi*exp(-4.*pi**2*t)
      
       return
       end
c*************************************************************
      real function fpsib*8(t)
      implicit real*8 (a-h,o-z)
c     fpsib(t) - function which returns values of the right-hand side
c     in Robin boundary conditions of the original differential
c     problem, at the end point x=b, at time t.
c     Number pi=3.14...
      pi=acos(-1.)

       fpsib= +4.*pi*exp(-4.*pi**2*t)
c      fpsib= +2.*pi*exp(-4.*pi**2*t)
       return
       end
c*************************************************************
      subroutine hcoeff(IAPP)
      implicit real*8 (a-h,o-z)
 
c     t,dt- current time, and time step
      common/t/ t,dt
 
c    sigma - parameter in the finite-difference scheme
      common/expimp/ sigma
 
c     M - number of nodes in the grid
      common/M/ M
 
c     VC, VN - arrays of volume of cells and nodes
      common/volumes/ VC(200), VN(200)
 
c     U    - array (1 to M), solution on the previous time level
      common/U/  U(200)
 
 
c     F - array (2 to M-1), values of the right-hand side of the differential
c                           equation in nodes
      common/f/ f(200)
 
c     =================================================================
c     W, C, E, PHI -  coefficients and the right-hand side
c     in finite-difference equations in node i
c
c                 W*U(i-1) + C*U(i) + E*U(i+1) = PFI
c
      Common/coefds/ w(200),c(200),e(200),phi(200)
c     =================================================================
c     Coefficients and the right hand side of
c     the implicit finite-difference scheme
c     for heat equations
 
      do i=1,M
      phi(i)=phi(i)
     * - (1.-sigma)*(w(i)*U(i-1)+C(i)*U(i)+e(i)*U(i+1))
      w(i)=sigma*w(i)
      e(i)=sigma*e(i)
      c(i)=sigma*c(i)
      end do
 
      do i=2,M-1
      phi(i)=phi(i) + U(i)/dt
      c(i)=c(i) + 1./dt
      end do

      if(IAPP.eq.2) then
      c(1)=c(1)+VN(1)/dt
      c(M)=c(M)+VN(M)/dt
      end if
 
 
      return
      end
c **********************************************************
 
      subroutine sphi(IAPP)
      implicit real*8 (a-h,o-z)
 
c     t,dt- current time, and time step
      common/t/ t,dt

c     M - number of nodes in the grid
      common/M/ M
 
c     VC, VN - arrays of the volume of cells and nodes
      common/volumes/ VC(200), VN(200)

c     U    - array (1 to M), solution for the previous time level
      common/U/  U(200)

      common/f/ f(200)
 
c     psia, psib - constants in the right-hand side of the Robin boundary 
c     conditions in the points a and b respectively.
      common/psi/ psia, psib
c     =================================================================
c     W, C, E, PHI -  coefficients and the right-hand side
c     in the finite-difference equations in node i
c
c                 W*U(i-1) + C*U(i) + E*U(i+1) = PFI
c
c     =================================================================
 
 
 
      Common/coefds/ w(200),c(200),e(200),phi(200)
 
c     =================================================================
c     Coefficients and the right hand side of
c     the implicit finite-difference scheme
c     for heat equations
 
      if(IAPP.eq.1) then
c     first order approximation for the Robin boundary condition
       phi(1) = psia
       else
c     second order approximation for the Robin boundary condition
       phi(1) = psia + VN(1)*(f(1)+u(1)/dt)
       end if
      do i=2,M-1
      phi(i)=f(i)
      end do
 
      if(IAPP.eq.1) then
c     first order approximation for the Robin boundary condition
       phi(M)=psib
       else
c     second order approximation for the Robin boundary condition
       phi(M)=psib + VN(M) * ( f(M) + u(M)/dt)
       end if
      return
      end
c ********************************************************************



